# Copyright 2021 Le Filament
# License AGPL-3.0 or later (https://www.gnu.org/licenses/agpl).

from odoo import _, fields, models
from odoo.exceptions import UserError


class ScopBordereauChangePaymentMode(models.TransientModel):
    _name = "scop.bordereau.change.payment.mode.wizard"
    _description = "Changement de mode de paiement"

    bordereau_id = fields.Many2one(
        comodel_name="scop.bordereau",
        string="Bordereau",
        readonly=True,
    )
    payment_mode_id = fields.Many2one(
        comodel_name="account.payment.mode",
        string="Nouveau mode de paiment",
    )
    change_type = fields.Selection(
        string="Type d'affectation",
        selection=[
            (
                "punctual",
                "Définir pour les échéances non payées de ce " "bordereau uniquement",
            ),
            ("all", "Définir pour toutes les prochaines échéances"),
        ],
        default="punctual",
        required=False,
    )

    # ------------------------------------------------------
    # Action Button
    # ------------------------------------------------------
    def action_change_payment_mode(self):
        bordereau_id = self.bordereau_id
        if self.change_type == "all":
            # Affecte le mode de paiement par défaut au contact
            bordereau_id.partner_id.update(
                {
                    "customer_payment_mode_id": self.payment_mode_id.id,
                }
            )
            # Recherche les bordereaux non payés
            bordereau_ids = self.env["scop.bordereau"].search([
                ("partner_id", "=", bordereau_id.partner_id.id),
                ("state", "not in", ["paid", "cancel"]),
            ])
            # Recherche les factures liées aux bordereaux non payées
            invoice_ids = bordereau_ids.mapped("invoice_ids").filtered(
                lambda i: i.payment_state not in ["paid", "reversed"])
        else:
            bordereau_ids = bordereau_id
            invoice_ids = bordereau_id.invoice_ids.filtered(
                lambda i: i.payment_state not in ["paid", "reversed"])
            if not invoice_ids:
                raise UserError(_(
                    "Tous les appels liés à ce bordereau sont payés, il n'est pas possible "
                    "de modifier le mode de paiement"
                ))

        # Met à jour le mode de paiement sur les bordereaux
        bordereau_ids.update({"payment_mode_id": self.payment_mode_id.id})
        # Met à jour le mode de paiement sur les factures et attribue le mandat
        invoice_ids.update({"payment_mode_id": self.payment_mode_id.id})
        invoice_ids.set_mandate()

        journal_ids = [
            self.env.user.company_id.contribution_journal_id.id,
            self.env.user.company_id.journal_fede_com_id.id,
            self.env.user.company_id.journal_fede_cae_id.id,
            self.env.user.company_id.journal_fede_indus_id.id,
            self.env.user.company_id.journal_ur_hdf_id.id,
            self.env.user.company_id.journal_ur_med_id.id,
        ]
        # Met à jour le mode de paiement sur les écritures comptables
        move_line_ids = self.env["account.move.line"].search(
            [
                ("move_id", "in", invoice_ids.ids),
                ("journal_id", "in", journal_ids),
                ("full_reconcile_id", "=", False),
                ("balance", "!=", 0),
                ("account_id.reconcile", "=", True),
                ("account_id.internal_type", "=", "receivable"),
            ]
        )
        move_line_ids.update(
            {
                "payment_mode_id": self.payment_mode_id.id,
            }
        )
        return {"type": "ir.actions.act_window_close"}
